# Auto-generated entities
from __future__ import annotations

from decimal import Decimal
from typing import Any

from base_repository import ConfigurableEntity, EntityConfig, KeyType
from pydantic import BaseModel
